@@ -1,4 +1,4 @@
-// HighOrderMeshOptimizer - Copyright (C) 2013-2019 UCLouvain-ULiege
+// HighOrderMeshOptimizer - Copyright (C) 2013-2025 UCLouvain-ULiege
 //
 // Permission is hereby granted, free of charge, to any person
 // obtaining a copy of this software and associated documentation
@@ -32,13 +32,13 @@
 
 #include "MEdge.h"
 #include "MFace.h"
+#include "nodalBasis.h"
 #include "fullMatrix.h"
 
 class MElement;
 class GEntity;
 class MVertex;
 struct IntPt;
-struct nodalBasis;
 
 typedef std::pair<MElement *, std::vector<MElement *> > PairMElemVecMElem;
 typedef std::vector<PairMElemVecMElem> VecPairMElemVecMElem;
@@ -201,7 +201,7 @@ namespace BoundaryLayerCurver {
 
 // BL in planar surface (always prefer this one if possible)
 void curve2DBoundaryLayer(VecPairMElemVecMElem &bndEl2column, SVector3 normal,
-                          const GEdge *edge = NULL);
+                          const GEdge *edge = nullptr);
 
 // BL on CAD surface
 void curve2DBoundaryLayer(VecPairMElemVecMElem &bndEl2column, const GFace *,
