@@ -1,4 +1,4 @@
-// HighOrderMeshOptimizer - Copyright (C) 2013-2019 UCLouvain-ULiege
+// HighOrderMeshOptimizer - Copyright (C) 2013-2025 UCLouvain-ULiege
 //
 // Permission is hereby granted, free of charge, to any person
 // obtaining a copy of this software and associated documentation
@@ -50,11 +50,14 @@
 #include "boundaryLayersData.h"
 #include "BoundaryLayerCurver.h"
 #include "FuncSpaceData.h"
+#include "nodalBasis.h"
 
 namespace {
 
-  typedef std::map<MEdge, std::vector<MElement *>, Less_Edge> MEdgeVecMEltMap;
-  typedef std::map<MFace, std::vector<MElement *>, Less_Face> MFaceVecMEltMap;
+  typedef std::map<MEdge, std::vector<MElement *>, MEdgeLessThan>
+    MEdgeVecMEltMap;
+  typedef std::map<MFace, std::vector<MElement *>, MFaceLessThan>
+    MFaceVecMEltMap;
 
   // Compute edge -> element connectivity (for 2D elements)
   void calcEdge2Elements(GEntity *entity, MEdgeVecMEltMap &ed2el)
@@ -100,12 +103,12 @@ namespace {
 
   inline void insertIfCurved(MElement *el, std::list<MElement *> &bndEl)
   {
-    static const double curvedTol =
-      1.e-3; // Tolerance to consider element as curved
+    // Tolerance to consider element as curved
+    static const double curvedTol = 1.e-3;
+
+    // Threshold in normal displacement to consider element as curved
+    const double normalDispCurved = curvedTol * el->getInnerRadius();
 
-    const double normalDispCurved =
-      curvedTol * el->getInnerRadius(); // Threshold in normal displacement to
-                                        // consider element as curved
     const int dim = el->getDim();
 
     // Compute unit normal to straight edge/face
@@ -222,12 +225,12 @@ namespace {
   {
     const double maxDP = std::cos(p.maxAngle);
 
-    MElement *el = 0;
+    MElement *el = nullptr;
 
     for(int iLayer = 0; iLayer < p.maxNumLayers; iLayer++) {
       std::vector<MElement *> newElts = ed2el[elBaseEd];
       if((iLayer > 0) && (newElts.size() < 2)) {
-        aboveElt = 0;
+        aboveElt = nullptr;
         break;
       }
       el = (newElts[0] == el) ? newElts[1] : newElts[0];
@@ -253,13 +256,13 @@ namespace {
     const double maxDP = std::cos(p.maxAngle);
     const double maxDPIn = std::cos(p.maxAngleInner);
 
-    MElement *el0 = 0, *el1 = 0;
+    MElement *el0 = nullptr, *el1 = nullptr;
 
     for(int iLayer = 0; iLayer < p.maxNumLayers; iLayer++) {
       // Get first element in layer
       std::vector<MElement *> newElts0 = ed2el[elBaseEd];
       if((iLayer > 0) && (newElts0.size() < 2)) {
-        aboveElt = 0;
+        aboveElt = nullptr;
         break;
       }
       el0 = (newElts0[0] == el1) ? newElts0[1] : newElts0[0];
@@ -275,7 +278,7 @@ namespace {
       // Get second element in layer
       std::vector<MElement *> newElts1 = ed2el[elTopEd0];
       if(newElts1.size() < 2) {
-        aboveElt = 0;
+        aboveElt = nullptr;
         break;
       }
       el1 = (newElts1[0] == el0) ? newElts1[1] : newElts1[0];
@@ -449,25 +452,25 @@ namespace {
       }
     }
 
-    // Build top face from max face (with right correspondance)
+    // Build top face from max face (with right correspondence)
     MVertex *maxVert[3] = {elMaxFace.getVertex(0), elMaxFace.getVertex(1),
                            elMaxFace.getVertex(2)};
-    std::vector<MVertex *> topVert(3, static_cast<MVertex *>(0));
-    for(int iBaseV = 0; iBaseV < 3;
-        iBaseV++) // Two vertices of elTopFace are those of elMaxFace coinciding
-                  // with elBaseFace
+    std::vector<MVertex *> topVert(3, static_cast<MVertex *>(nullptr));
+    // Two vertices of elTopFace are those of elMaxFace coinciding with
+    // elBaseFace
+    for(int iBaseV = 0; iBaseV < 3; iBaseV++)
       for(int iMaxV = 0; iMaxV < 3; iMaxV++)
         if(elBaseFace.getVertex(iBaseV) == maxVert[iMaxV]) {
           topVert[iBaseV] = maxVert[iMaxV];
-          maxVert[iMaxV] = 0;
+          maxVert[iMaxV] = nullptr;
         }
-    MVertex *thirdMaxVert = (maxVert[0] != 0) ?
-                              maxVert[0] : // Set last vertex of elTopFace as
-                                           // remaining vertex in elMaxFace
-                              (maxVert[1] != 0) ? maxVert[1] : maxVert[2];
-    if(topVert[0] == 0)
+    // Set last vertex of elTopFace as remaining vertex in elMaxFace
+    MVertex *thirdMaxVert = (maxVert[0] != nullptr) ? maxVert[0] :
+                            (maxVert[1] != nullptr) ? maxVert[1] :
+                                                      maxVert[2];
+    if(topVert[0] == nullptr)
       topVert[0] = thirdMaxVert;
-    else if(topVert[1] == 0)
+    else if(topVert[1] == nullptr)
       topVert[1] = thirdMaxVert;
     else
       topVert[2] = thirdMaxVert;
@@ -482,13 +485,13 @@ namespace {
     const double maxDP = std::cos(p.maxAngle);
     const double maxDPIn = std::cos(p.maxAngleInner);
 
-    MElement *el0 = 0, *el1 = 0, *el2 = 0;
+    MElement *el0 = nullptr, *el1 = nullptr, *el2 = nullptr;
 
     for(int iLayer = 0; iLayer < p.maxNumLayers; iLayer++) {
       // Get first element in layer
       std::vector<MElement *> newElts0 = face2el[elBaseFace];
       if((iLayer > 0) && (newElts0.size() < 2)) {
-        aboveElt = 0;
+        aboveElt = nullptr;
         break;
       }
       el0 = (newElts0[0] == el2) ? newElts0[1] : newElts0[0];
@@ -505,7 +508,7 @@ namespace {
       // Get second element in layer
       std::vector<MElement *> newElts1 = face2el[elTopFace0];
       if(newElts1.size() < 2) {
-        aboveElt = 0;
+        aboveElt = nullptr;
         break;
       }
       el1 = (newElts1[0] == el0) ? newElts1[1] : newElts1[0];
@@ -520,7 +523,7 @@ namespace {
       // Get third element in layer
       std::vector<MElement *> newElts2 = face2el[elTopFace1];
       if(newElts2.size() < 2) {
-        aboveElt = 0;
+        aboveElt = nullptr;
         break;
       }
       el2 = (newElts2[0] == el1) ? newElts2[1] : newElts2[0];
@@ -554,12 +557,12 @@ namespace {
   {
     const double maxDP = std::cos(p.maxAngle);
 
-    MElement *el = 0;
+    MElement *el = nullptr;
 
     for(int iLayer = 0; iLayer < p.maxNumLayers; iLayer++) {
       std::vector<MElement *> newElts = face2el[elBaseFace];
       if((iLayer > 0) && (newElts.size() < 2)) {
-        aboveElt = 0;
+        aboveElt = nullptr;
         break;
       }
       el = (newElts[0] == el) ? newElts[1] : newElts[0];
@@ -596,7 +599,7 @@ namespace {
     el->getFaceInfo(baseFace, iFirstElFace, iDum, iDum);
     el->getFaceVertices(iFirstElFace, baseVert);
     MFace elBaseFace(baseVert[0], baseVert[1], baseVert[2],
-                     (nbBaseFaceVert == 3) ? 0 : baseVert[3]);
+                     (nbBaseFaceVert == 3) ? nullptr : baseVert[3]);
 
     // Sweep column upwards by choosing largest faces in each element
     if(nbBaseFaceVert == 3) {
@@ -610,8 +613,7 @@ namespace {
       getColumnPrismHex(TYPE_HEX, face2el, p, elBaseFace, blob, aboveElt);
     else
       return false; // Not a BL
-    if(blob.size() == 0)
-      return false; // Could not find column (may not be a BL)
+    if(blob.size() == 0) return false; // Could not find column (may not be BL)
 
     // Create top face of column with last face vertices
     topPrimVert.resize(nbBaseFaceVert);
@@ -688,8 +690,7 @@ namespace {
       fprintf(fp, "$MeshFormat\n2.2 0 8\n$EndMeshFormat\n");
       fprintf(fp, "$Nodes\n");
       fprintf(fp, "%d\n", vert_.size());
-      for(std::set<MVertex *>::iterator itV = vert_.begin(); itV != vert_.end();
-          ++itV) {
+      for(auto itV = vert_.begin(); itV != vert_.end(); ++itV) {
         SPoint3 p = (*itV)->point();
         fprintf(fp, "%i %g %g %g\n", (*itV)->getNum(), p.x(), p.y(), p.z());
       }
@@ -755,9 +756,9 @@ namespace {
     for(int iV = 0; iV < nbVert; iV++) {
       MVertex *vert = bndElt->getVertex(iV);
       double tCAD;
-      if(vert->onWhat() == geomEnt) // If HO vertex, ...
-        vert->getParameter(
-          0, tCAD); // ... get stored param. coord. (can be only line).
+      // If HO vertex, get stored param. coord. (can be only line).
+      if(vert->onWhat() == geomEnt)
+        vert->getParameter(0, tCAD);
       else { // Otherwise, get param. coord. from CAD.
         if(ge->getBeginVertex() &&
            (ge->getBeginVertex()->mesh_vertices[0] == vert))
@@ -842,8 +843,8 @@ namespace {
       distSq0 = distSq;
     }
 
-    if(!converged || (u < uMin + TOL) ||
-       (u > uMax - TOL)) { // Set to mid-point if not converged
+    // Set to mid-point if not converged
+    if(!converged || (u < uMin + TOL) || (u > uMax - TOL)) {
       u = uMin + 0.5 * du;
       vert->setParameter(0, u);
       GPoint gp = ge->point(u);
@@ -975,7 +976,7 @@ namespace {
     metaElt.setCurvedTop(deformFact);
     std::set<MVertex *> movedVertDum;
     curveElement(metaElt, movedVertDum, lastElt);
-    if(aboveElt == 0) return 1.;
+    if(aboveElt == nullptr) return 1.;
     double minJacDet, maxJacDet;
     jacobianBasedQuality::minMaxJacobianDeterminant(aboveElt, minJacDet,
                                                     maxJacDet);
@@ -1013,11 +1014,9 @@ namespace {
       // Bisection on displacement to avoid breaking the element above if
       // required
       if((p.curveOuterBL == FastCurvingParameters::OUTER_CURVECONSERVATIVE) &&
-         (qualDeformMax <
-          MINQUAL)) { // Max deformation makes element above invalid
-        for(int iter = 0; iter < MAXITER;
-            iter++) { // Bisection to find max. deformation that makes element
-                      // above valid
+         (qualDeformMax < MINQUAL)) { // Max deform. makes elt. above invalid
+        // Bisection to find max. deformation that makes element above valid
+        for(int iter = 0; iter < MAXITER; iter++) {
           const double deformMid = 0.5 * (deformMin + deformMax);
           const double qualDeformMid =
             curveAndMeasureAboveEl(metaElt, lastElt, aboveElt, deformMid);
@@ -1054,7 +1053,7 @@ namespace {
     bool foundCol;
     std::vector<MVertex *> baseVert, topPrimVert;
     std::vector<MElement *> blob;
-    MElement *aboveElt = 0;
+    MElement *aboveElt = nullptr;
     if(bndType == TYPE_LIN) { // 1D boundary
       MVertex *vb0 = bndElt->getVertex(0);
       MVertex *vb1 = bndElt->getVertex(1);
@@ -1073,19 +1072,19 @@ namespace {
         metaElType = TYPE_HEX;
       }
       else {
-        vb3 = 0;
+        vb3 = nullptr;
         metaElType = TYPE_PRI;
       }
       MFace baseFace(vb0, vb1, vb2, vb3);
       foundCol = getColumn3D(face2el, p, baseFace, baseVert, topPrimVert, blob,
                              aboveElt);
     }
-    if(!foundCol || blob.empty())
-      return; // Skip bnd. el. if top vertices not found
-    DbgOutputCol dbgOutCol;
-    dbgOutCol.addBlob(blob);
-    //  dbgOutCol.write("col_KO", bndElt->getNum());
-    if(aboveElt == 0)
+    // Skip bnd. el. if top vertices not found
+    if(!foundCol || blob.empty()) return;
+    // DbgOutputCol dbgOutCol;
+    // dbgOutCol.addBlob(blob);
+    // dbgOutCol.write("col_KO", bndElt->getNum());
+    if(aboveElt == nullptr)
       std::cout << "DBGTT: aboveElt = 0 for bnd. elt. " << bndElt->getNum()
                 << std::endl;
     curveColumn(p, ent, bndEnt, metaElType, baseVert, topPrimVert, aboveElt,
@@ -1105,13 +1104,13 @@ namespace {
     std::vector<std::pair<MElement *, std::vector<MElement *> > > bndEl2column;
     std::vector<MElement *> aboveElements;
 
-    std::list<MElement *>::iterator it = bndElts.begin();
+    auto it = bndElts.begin();
     while(it != bndElts.end()) {
       MElement *bndEl = *it;
       const int bndType = bndEl->getType();
       bool foundCol;
       std::vector<MVertex *> baseVert, topPrimVert;
-      MElement *aboveElt = 0;
+      MElement *aboveElt = nullptr;
 
       if(bndType == TYPE_LIN) { // 1D boundary
         MVertex *vb0 = bndEl->getVertex(0);
@@ -1120,7 +1119,7 @@ namespace {
 
         // Check if baseEd is adjacent to an element of the face
         // (the contrary can happen with degenerate edge, see fix b91a1b822)
-        MEdgeVecMEltMap::iterator myit = ed2el.find(baseEd);
+        auto myit = ed2el.find(baseEd);
         if(myit == ed2el.end()) {
           ++it;
           continue;
@@ -1129,7 +1128,7 @@ namespace {
         std::vector<MElement *> vec;
         bndEl2column.push_back(
           std::make_pair(bndEl, std::vector<MElement *>()));
-        aboveElements.push_back(NULL);
+        aboveElements.push_back(nullptr);
         foundCol =
           getColumn2D(ed2el, p, baseEd, baseVert, topPrimVert,
                       bndEl2column.back().second, aboveElements.back());
@@ -1138,12 +1137,12 @@ namespace {
         MVertex *vb0 = bndEl->getVertex(0);
         MVertex *vb1 = bndEl->getVertex(1);
         MVertex *vb2 = bndEl->getVertex(2);
-        MVertex *vb3 = NULL;
+        MVertex *vb3 = nullptr;
         if(bndType == TYPE_QUA) vb3 = bndEl->getVertex(3);
         MFace baseFace(vb0, vb1, vb2, vb3);
         bndEl2column.push_back(
           std::make_pair(bndEl, std::vector<MElement *>()));
-        aboveElements.push_back(NULL);
+        aboveElements.push_back(nullptr);
         foundCol =
           getColumn3D(face2el, p, baseFace, baseVert, topPrimVert,
                       bndEl2column.back().second, aboveElements.back());
@@ -1177,6 +1176,19 @@ namespace {
     //  else curve3DBoundaryLayer(bndEl2column);
   }
 
+  std::string entName(const GEntity *ge)
+  {
+    std::stringstream stream;
+    switch(ge->dim()) {
+    case 0: stream << "point "; break;
+    case 1: stream << "curve "; break;
+    case 2: stream << "surface "; break;
+    case 3: stream << "volume "; break;
+    }
+    stream << ge->tag();
+    return stream.str();
+  }
+
   void curveMeshFromBnd(MEdgeVecMEltMap &ed2el, MFaceVecMEltMap &face2el,
                         GEntity *ent, GEntity *bndEnt,
                         const FastCurvingParameters &p, SVector3 const normal)
@@ -1204,8 +1216,7 @@ namespace {
           insertIfCurved(gFace->quadrangles[i], bndEl);
     }
     else
-      Msg::Error("Cannot process model entity %i of dim %i", bndEnt->tag(),
-                 bndEnt->dim());
+      Msg::Error("Cannot process %s in fast curving", entName(bndEnt).c_str());
 
     if(p.thickness) {
       getColumnsAndcurveBoundaryLayer(ed2el, face2el, ent, bndEnt, bndEl, p,
@@ -1216,8 +1227,8 @@ namespace {
     // Loop over boundary elements to curve them by columns
     DbgOutputMeta dbgOut;
     std::set<MVertex *> movedVert;
-    for(std::list<MElement *>::iterator itBE = bndEl.begin();
-        itBE != bndEl.end(); itBE++) // Loop over bnd. elements
+    for(auto itBE = bndEl.begin(); itBE != bndEl.end();
+        itBE++) // Loop over bnd. elements
       curveMeshFromBndElt(ed2el, face2el, ent, bndEnt, *itBE, movedVert, p,
                           dbgOut);
     //  dbgOut.write("meta-elements", bndEnt->tag());
@@ -1231,8 +1242,7 @@ namespace {
     // inspired from curveMeshFromBnd and curveMeshFromBndElt
 
     if(bndEnt->dim() != 2) {
-      Msg::Error("Cannot process model entity %i of dim %i", bndEnt->tag(),
-                 bndEnt->dim());
+      Msg::Error("Cannot process %s in fast curving", entName(bndEnt).c_str());
       return;
     }
 
@@ -1243,7 +1253,7 @@ namespace {
     for(std::size_t i = 0; i < gFace->quadrangles.size(); i++)
       bndElts.push_back(gFace->quadrangles[i]);
 
-    std::list<MElement *>::iterator it = bndElts.begin();
+    auto it = bndElts.begin();
     while(it != bndElts.end()) {
       MElement *bndEl = *it;
       const int bndType = bndEl->getType();
@@ -1253,7 +1263,7 @@ namespace {
       MVertex *vb0 = bndEl->getVertex(0);
       MVertex *vb1 = bndEl->getVertex(1);
       MVertex *vb2 = bndEl->getVertex(2);
-      MVertex *vb3 = NULL;
+      MVertex *vb3 = nullptr;
       if(bndType == TYPE_QUA) vb3 = bndEl->getVertex(3);
       MFace baseFace(vb0, vb1, vb2, vb3);
       bndEl2column.push_back(std::make_pair(bndEl, std::vector<MElement *>()));
@@ -1289,8 +1299,10 @@ void HighOrderMeshFastCurving(GModel *gm, FastCurvingParameters &p,
     int n = fields->getNumBoundaryLayerFields();
     for(int i = 0; i < n; ++i) {
       Field *bl_field = fields->get(fields->getBoundaryLayerField(i));
-      if(bl_field == NULL) continue;
-      blFields.push_back(dynamic_cast<BoundaryLayerField *>(bl_field));
+      if(!bl_field) continue;
+      BoundaryLayerField *blf = dynamic_cast<BoundaryLayerField *>(bl_field);
+      if(!blf) continue;
+      blFields.push_back(blf);
     }
   }
 
@@ -1346,7 +1358,7 @@ void HighOrderMeshFastCurving(GModel *gm, FastCurvingParameters &p,
       continue; // Skip if BL info is required but there is none
 
     // Compute edge/face -> elt. connectivity
-    Msg::Info("Computing connectivity for entity %i...", gEnt->tag());
+    Msg::Info("Computing connectivity for %s...", entName(gEnt).c_str());
     MEdgeVecMEltMap ed2el;
     MFaceVecMEltMap face2el;
     if(p.dim == 2)
@@ -1361,16 +1373,15 @@ void HighOrderMeshFastCurving(GModel *gm, FastCurvingParameters &p,
       GEntity *&bndEnt = bndEnts[iBndEnt];
       if(p.onlyVisible && !bndEnt->getVisibility())
         continue; // Skip if "only visible" required and entity is invisible
-      if(!blBndEnts
-            .empty() && // Skip if there is BL info but not on this boundary
-         (blBndEnts.find(bndEnt) == blBndEnts.end()))
+      // Skip if there is BL info but not on this boundary
+      if(!blBndEnts.empty() && (blBndEnts.find(bndEnt) == blBndEnts.end()))
         continue;
       const GEntity::GeomType bndType = bndEnt->geomType();
       if((bndType == GEntity::Line) || (bndType == GEntity::Plane))
         continue; // Skip if boundary is straight
       if(p.dim == 2 || !p.thickness) {
-        Msg::Info("Curving elements in surface %d for boundary edge %d...",
-                  gEnt->tag(), bndEnt->tag());
+        Msg::Info("Curving elements in %s for boundary %s...",
+                  entName(gEnt).c_str(), entName(bndEnt).c_str());
         curveMeshFromBnd(ed2el, face2el, gEnt, bndEnt, p, normal);
       }
       else
@@ -1378,7 +1389,7 @@ void HighOrderMeshFastCurving(GModel *gm, FastCurvingParameters &p,
     }
 
     if(p.thickness && p.dim == 3 && bndEnts.size()) {
-      Msg::Info("Curving elements in volume %d...", gEnt->tag());
+      Msg::Info("Curving elements in %s...", entName(gEnt).c_str());
       curve3DBoundaryLayer(bndEl2column, (GFace *)bndEnts[0]);
     }
   }
